/*
 * System Interface Library for games
 * Copyright (c) 2011 Andrew Church <achurch@achurch.org>
 * Released under the GNU GPL version 3 or later; NO WARRANTY is provided.
 * See the file COPYING.txt for details.
 *
 * src/test/sysdep/ios/graphics.c: iOS-specific graphics tests.
 */

#ifdef INCLUDE_TESTS  // To the end of the file.

#include "src/base.h"
#include "src/graphics.h"
#define IN_SYSDEP  // So we get the real sys_time_*() functions.
#include "src/sysdep.h"
#include "src/test/base.h"
#include "src/time.h"

/*************************************************************************/
/****************************** Test runner ******************************/
/*************************************************************************/

DEFINE_GENERIC_TEST_RUNNER(test_ios_graphics)

/*************************************************************************/
/***************************** Test routines *****************************/
/*************************************************************************/

TEST(test_frame_interval)
{
    ASSERT(graphics_init());
    const double time_unit = sys_time_init();

    CHECK_TRUE(graphics_set_display_attr("vsync", 1));
    CHECK_TRUE(graphics_set_display_attr("frame_interval", 1));
    CHECK_TRUE(graphics_set_display_mode(graphics_device_width(),
                                         graphics_device_height()));
    graphics_start_frame();
    graphics_clear(0, 0, 0);
    graphics_finish_frame();

    double start = sys_time_now() / time_unit;
    for (int i = 0; i < 10; i++) {
        graphics_start_frame();
        graphics_clear((i+1)/10.0f, (i+1)/10.0f, (i+1)/10.0f);
        graphics_finish_frame();
    }
    const double interval1_time = (sys_time_now() / time_unit) - start;

    /* This should take effect immediately. */
    CHECK_TRUE(graphics_set_display_attr("frame_interval", 2));

    start = sys_time_now() / time_unit;
    for (int i = 0; i < 10; i++) {
        graphics_start_frame();
        graphics_clear((9-i)/10.0f, (9-i)/10.0f, (9-i)/10.0f);
        graphics_finish_frame();
    }
    const double interval2_time = (sys_time_now() / time_unit) - start;

    /* Cleanup and reinit should reset the frame interval to 1. */
    graphics_cleanup();
    graphics_init();
    CHECK_TRUE(graphics_set_display_mode(graphics_device_width(),
                                         graphics_device_height()));
    graphics_start_frame();
    graphics_clear(0, 0, 0);
    graphics_finish_frame();

    start = sys_time_now() / time_unit;
    for (int i = 0; i < 10; i++) {
        graphics_start_frame();
        graphics_clear((i+1)/10.0f, (i+1)/10.0f, (i+1)/10.0f);
        graphics_finish_frame();
    }
    const double interval1_time2 = (sys_time_now() / time_unit) - start;

    graphics_cleanup();

    CHECK_DOUBLERANGE(interval2_time, interval1_time*1.8, interval1_time*2.2);
    CHECK_DOUBLERANGE(interval1_time2, interval1_time*0.9, interval1_time*1.1);

    return 1;
}

/*************************************************************************/
/*************************************************************************/

#endif  // INCLUDE_TESTS

/*
 * Local variables:
 *   c-file-style: "stroustrup"
 *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
 *   indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
 */
